/*****************************************
 * crée un tableau d'entier avec des nombres pseudo-aléatoires
 * entre m et et M-1
 *
 * entrée :
 *   - m, M : bornes des nombres pseudo-aléatoires
 *   - n : taille du tableau
 *
 * sortie :
 *   - tableau d'entiers de taille n
 ******************************************/
 int [] tabAleatoire(int m, int M, int n) {
  int [] t = new int[n];
  
  for(int i = 0; i < n; i++)
    t[i] = int(random(m, M));
    
  return t;
 }
 
/*****************************************
 * crée un tableau d'entier avec des nombres pseudo-aléatoires
 * dans l'ordre croissant
 *
 * entrée :
 *   - p : probabilité d'accepter le nombre
 *   - n : taille du tableau
 *
 * sortie :
 *   - tableau d'entiers de taille n
 ******************************************/
 int [] tabAleatoireCroissant(float p, int n) {
  int [] t = new int[n];
  
  int val = 0;
  
  for(int i = 0; i < n; i++) {
    while (p < random(1.0))
      val++;
      
    t[i] = val;
  }
    
  return t;
 }
 
/*****************************************
 * affiche un tableau d'entier
 *
 * entrée :
 *   - t : tableau
 *
 * sortie :
 *   - aucune seulement sortie écran
 ******************************************/
 void afficher(int [] t) {
  for(int i = 0; i < t.length; i++)
    print(t[i] + " ");
 }
 
/*****************************************
 * recherche d'un nombre dans un tableau de manière itérative
 *
 * entrée :
 *   - x : nombre à rechercher
 *   - t : tableau où le nombre est à rechercher
 *
 * sortie :
 *   - booléen : vrai ssi le nombre est dans le tableau
 ******************************************/
 boolean rechercheIter(int x, int [] t) {
   int i = 0;
   
   while (i < t.length && t[i] != x)
     i++;
     
   return i < t.length;
 }
 
/*****************************************
 * recherche d'un nombre dans un tableau de manière dichotomique
 *
 * entrée :
 *   - x : nombre à rechercher
 *   - t : tableau où le nombre est à rechercher
 *
 * sortie :
 *   - booléen : vrai ssi le nombre est dans le tableau
 ******************************************/
 boolean rechercheDichotomique(int x, int [] t) {
   int a = 0;
   int b = t.length - 1;
   int c = (a + b) / 2;
   
   while (a <= b && t[c] != x) {
     if (t[c] < x)
       a = c + 1;
     else
       b = c - 1; 
     c = (a + b) / 2;
   }
     
   return a <= b;
 }
 
/*****************************************
 * denombrer le nombre d'apparition d'un nombre dans un tableau
 *
 * entrée :
 *   - x : nombre à dénombrer
 *   - t : tableau où le nombre est à rechercher
 *
 * sortie :
 *   - nombre d'occurence
 ******************************************/
 int denombrer(int x, int [] t) {
   int c = 0;
   
   for(int i = 0; i < t.length; i++)
     if (t[i] == x)
       c++;
       
   return c;
 }
 
/*****************************************
 * denombrer en parallèle les nombres pairs
 *
 * entrée :
 *   - t1 : tableau où le nombre est à rechercher
 *   - t2 : tableau où le nombre est à rechercher
 *
 * sortie :
 *   - nombre d'occurences de nombre pairs en parallèle
 ******************************************/
 int denombrerParallele(int [] t1, int [] t2) {
   int c = 0;
   
   for(int i = 0; i < t1.length; i++)
     if (t1[i] % 2 == 0 && t2[i] % 2 == 0)
       c++;
       
   return c;
 }
  
 void setup() {
   int [] tab = tabAleatoire(0, 10, 10);
   
   afficher(tab);
   println();
   
   println(rechercheIter(2, tab));
   println(rechercheIter(12, tab));

  tab = tabAleatoireCroissant(0.3, 10);
  
   afficher(tab);
   println();
   
   println(rechercheDichotomique(2, tab));
   println(rechercheDichotomique(12, tab));
 
   int [] t = { 3, 2, 45, 2, 10, 2 };
   println(denombrer(2, t));
   println(denombrer(21, t));

   int [] t2 = { 13, 12, 12, 20, 0, 23 };
   println(denombrerParallele(t, t2));
 }
